package edu.berkeley.slipway.gui;

import com.atmel.fpslic.*;
import static com.atmel.fpslic.FpslicConstants.*;
import static com.atmel.fpslic.FpslicUtil.*;
import edu.berkeley.slipway.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.color.*;
import org.ibex.util.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

public class GuiConstants {
    public static int SIZE = 92;
    public static int RINGS = 3;
    public static int BEVEL = 5;
    public static int CORE_SIZE = 64;
    public static int CORE_OFFSET = 10;
    public static int HOFF = 52;

    public static int RED  = 0xff0000;
    public static int BLUE = 0x0000ff;
    //public static int LIGHTRED  = 0xff0000;
    //public static int LIGHTBLUE = 0x0000ff;
    public static int LIGHTRED  = 0xffbbbb;
    public static int LIGHTBLUE = 0xbbbbff;
    public static int GREEN = 0x006b33;
    public static int WHITE = 0xffffff;
    public static int BLACK = 0x000000;
    public static int ORANGE = 0xff7f00;
    public static int MAGENTA = 0xff00ff;

    public static final P YLUT_OUTPUT_POINT  = new P(SIZE-CORE_OFFSET-CORE_SIZE+51 - 2, CORE_OFFSET + 41 - 3);
    public static final P XLUT_OUTPUT_POINT  = new P(SIZE-CORE_OFFSET-CORE_SIZE+17 - 2, CORE_OFFSET + 41 - 3);

    public static final String[] XLUT_EQUATIONS = new String[] {
        "0",
        "0",
        "~t x ~y",
        "~t ~y",
        "~t ~x y",
        "~t ~x",
        "~t (~x y + x ~y)",
        "~t (~x + ~y)",
        "~t x y",
        "~t x y",
        "~t x",
        "~t x",
        "~t y",
        "~t y",
        "~t (x + y)",
        "~t",
        "t ~x ~y",
        "~x ~y",
        "~y (~t x + t ~x)",
        "~y (~t + ~x)",
        "~x (~t y + t ~y)",
        "~x (~t + ~y)",
        "~t (~x y + x ~y) + t ~x ~y",
        "~t (~x + ~y) + ~x ~y",
        "t ~x ~y + ~t x y",
        "~x ~y + ~t x y",
        "~t x + t ~x ~y",
        "~x ~y + ~t x",
        "~t y + t ~x ~y",
        "~x ~y + ~t y",
        "~t (x + y) + t ~x ~y",
        "~t + ~x ~y",
        "t x ~y",
        "t x ~y",
        "x ~y",
        "x ~y",
        "t x ~y + ~t ~x y",
        "~t ~x + t x ~y",
        "x ~y + ~t ~x y",
        "x ~y + ~t ~x",
        "x (~t y + t ~y)",
        "x (~t y + t ~y)",
        "x (~t + ~y)",
        "x (~t + ~y)",
        "~t y + t x ~y",
        "~t y + t x ~y",
        "x ~y + ~t y",
        "~t + x ~y",
        "t ~y",
        "t ~y",
        "~y (t + x)",
        "~y",
        "t ~y + ~t ~x y",
        "t ~y + ~t ~x",
        "~y (t + x) + ~t ~x y",
        "~y + ~t ~x",
        "t ~y + ~t x y",
        "t ~y + ~t x y",
        "t ~y + ~t x",
        "~y + ~t x",
        "t ~y + ~t y",
        "t ~y + ~t y",
        "(t y + ~t ~x ~y)",
        "(t y)",
        "t ~x y",
        "t ~x y",
        "~t x ~y + t ~x y",
        "~t ~y + t ~x y",
        "~x y",
        "~x y",
        "~x y + ~t x ~y",
        "~t ~y + ~x y",
        "y (~t x + t ~x)",
        "y (~t x + t ~x)",
        "~t x + t ~x y",
        "~t x + t ~x y",
        "y (~t + ~x)",
        "y (~t + ~x)",
        "~x y + ~t x",
        "~t + ~x y",
        "t ~x",
        "t ~x",
        "t ~x + ~t x ~y",
        "~t ~y + t ~x",
        "~x (t + y)",
        "~x",
        "~x (t + y) + ~t x ~y",
        "~x + ~t ~y",
        "t ~x + ~t x y",
        "t ~x + ~t x y",
        "t ~x + ~t x",
        "t ~x + ~t x",
        "~t y + t ~x",
        "~x + ~t y",
        "~(t x + ~t ~x ~y)",
        "~(t x)",
        "t (~x y + x ~y)",
        "t (~x y + x ~y)",
        "x ~y + t ~x y",
        "x ~y + t ~x y",
        "~x y + t x ~y",
        "~x y + t x ~y",
        "x ~y + ~x y",
        "x ~y + ~x y",
        "t (~x y + x ~y) + ~t x y",
        "t (~x y + x ~y) + ~t x y",
        "x (~t + ~y) + t ~x y",
        "x (~t + ~y) + t ~x y",
        "y (~t + ~x) + t x ~y",
        "y (~t + ~x) + t x ~y",
        "~(~x ~y + t x y)",
        "~(~x ~y + t x y)",
        "t (~x + ~y)",
        "t (~x + ~y)",
        "x ~y + t ~x",
        "~y + t ~x",
        "t ~y + ~x y",
        "~x + t ~y",
        "~(x y + ~t ~x ~y)",
        "~(x y)",
        "t (~x + ~y) + ~t x y",
        "t (~x + ~y) + ~t x y",
        "~(~t ~x + t x y)",
        "~(~t ~x + t x y)",
        "~(~t ~y + t x y)",
        "~(~t ~y + t x y)",
        "~(~t ~x ~y + t x y)",
        "~(t x y)",
        "t x y",
        "~t ~x ~y + t x y",
        "~t ~y + t x y",
        "~t ~y + t x y",
        "~t ~x + t x y",
        "~t ~x + t x y",
        "~t (~x + ~y) + t x y",
        "~(t (~x + ~y) + ~t x y)",
        "x y",
        "x y + ~t ~x ~y",
        "~t ~y + x y",
        "~(t ~y + ~x y)",
        "y (~t + x)",
        "~(x ~y + t ~x)",
        "~t + x y",
        "~(t (~x + ~y))",
        "~x ~y + t x y",
        "~x ~y + t x y",
        "~y (~t + ~x) + t x y",
        "~(y (~t + ~x) + t x ~y)",
        "~x (~t + ~y) + t x y",
        "~(x (~t + ~y) + t ~x y)",
        "~(t (~x y + x ~y) + ~t x y)",
        "~(t (~x y + x ~y) + ~t x y)",
        "~x ~y + x y",
        "~(x ~y + ~x y)",
        "~(~x y + t x ~y)",
        "~(~x y + t x ~y)",
        "~(x ~y + t ~x y)",
        "~(x ~y + t ~x y)",
        "~(t (~x y + x ~y))",
        "~(t (~x y + x ~y))",
        "t x",
        "t x + ~t ~x ~y",
        "x (t + ~y)",
        "~(~t y + t ~x)",
        "~t ~x + t x",
        "~(t ~x + ~t x)",
        "~(t ~x + ~t x y)",
        "~(t ~x + ~t x y)",
        "x (t + y)",
        "~(~x (t + y) + ~t x ~y)",
        "x",
        "~(~x (t + y))",
        "~t y + t x",
        "~(t ~x + ~t x ~y)",
        "~(t ~x)",
        "~(t ~x)",
        "~x ~y + t x",
        "~(~x y + ~t x)",
        "~y + t x",
        "~(y (~t + ~x))",
        "~(~t x + t ~x y)",
        "~(~t x + t ~x y)",
        "~(y (~t x + t ~x))",
        "~(y (~t x + t ~x))",
        "t ~y + x y",
        "~(~x y + ~t x ~y)",
        "~(~x y)",
        "~(~x y)",
        "~(~t ~y + t ~x y)",
        "~(~t x ~y + t ~x y)",
        "~(t ~x y)",
        "~(t ~x y)",
        "t y",
        "t y + ~t ~x ~y",
        "~t ~y + t y",
        "~(t ~y + ~t y)",
        "y (t + ~x)",
        "~(t ~y + ~t x)",
        "~(t ~y + ~t x y)",
        "~(t ~y + ~t x y)",
        "y (t + x)",
        "~(~y (t + x) + ~t ~x y)",
        "~(t ~y + ~t ~x)",
        "~(t ~y + ~t ~x y)",
        "y",
        "~(~y (t + x))",
        "~(t ~y)",
        "~(t ~y)",
        "~x ~y + t y",
        "~(x ~y + ~t y)",
        "~(~t y + t x ~y)",
        "~(~t y + t x ~y)",
        "~x + t y",
        "~(x (~t + ~y))",
        "~(x (~t y + t ~y))",
        "~(x (~t y + t ~y))",
        "x y + t ~x",
        "~(x ~y + ~t ~x y)",
        "~(~t ~x + t x ~y)",
        "~(t x ~y + ~t ~x y)",
        "~(x ~y)",
        "~(x ~y)",
        "~(t x ~y)",
        "~(t x ~y)",
        "t (x + y)",
        "~(~t (x + y) + t ~x ~y)",
        "x ~y + t y",
        "~(~t y + t ~x ~y)",
        "~x y + t x",
        "~(~t x + t ~x ~y)",
        "~(~x ~y + ~t x y)",
        "~(t ~x ~y + ~t x y)",
        "t (x + y) + x y",
        "~(~t (~x y + x ~y) + t ~x ~y)",
        "x + t y",
        "~(~x (~t y + t ~y))",
        "y + t x",
        "~(~y (~t x + t ~x))",
        "~(~x ~y)",
        "~(t ~x ~y)",
        "t",
        "~(~t (x + y))",
        "~(~t y)",
        "~(~t y)",
        "~(~t x)",
        "~(~t x)",
        "~(~t x y)",
        "~(~t x y)",
        "t + x y",
        "~(~t (~x y + x ~y))",
        "~(~t ~x)",
        "~(~t ~x y)",
        "~(~t ~y)",
        "~(~t x ~y)",
        "1",
        "1"
    };

    public static final String[] YLUT_EQUATIONS = new String[] {
        "0",
        "0",
        "~t ~x y",
        "~t ~x",
        "~t x ~y",
        "~t ~y",
        "~t (~x y + x ~y)",
        "~t (~x + ~y)",
        "~t x y",
        "~t x y",
        "~t y",
        "~t y",
        "~t x",
        "~t x",
        "~t (x + y)",
        "~t",
        "t ~x ~y",
        "~x ~y",
        "~x (~t y + t ~y)",
        "~x (~t + ~y)",
        "~y (~t x + t ~x)",
        "~y (~t + ~x)",
        "~t (~x y + x ~y) + t ~x ~y",
        "~t (~x + ~y) + ~x ~y",
        "t ~x ~y + ~t x y",
        "~x ~y + ~t x y",
        "~t y + t ~x ~y",
        "~x ~y + ~t y",
        "~t x + t ~x ~y",
        "~x ~y + ~t x",
        "~t (x + y) + t ~x ~y",
        "~t + ~x ~y",
        "t ~x y",
        "t ~x y",
        "~x y",
        "~x y",
        "~t x ~y + t ~x y",
        "~t ~y + t ~x y",
        "~x y + ~t x ~y",
        "~t ~y + ~x y",
        "y (~t x + t ~x)",
        "y (~t x + t ~x)",
        "y (~t + ~x)",
        "y (~t + ~x)",
        "~t x + t ~x y",
        "~t x + t ~x y",
        "~x y + ~t x",
        "~t + ~x y",
        "t ~x",
        "t ~x",
        "~x (t + y)",
        "~x",
        "t ~x + ~t x ~y",
        "~t ~y + t ~x",
        "~x (t + y) + ~t x ~y",
        "~x + ~t ~y",
        "t ~x + ~t x y",
        "t ~x + ~t x y",
        "~t y + t ~x",
        "~x + ~t y",
        "t ~x + ~t x",
        "t ~x + ~t x",
        "~(t x + ~t ~x ~y)",
        "~(t x)",
        "t x ~y",
        "t x ~y",
        "t x ~y + ~t ~x y",
        "~t ~x + t x ~y",
        "x ~y",
        "x ~y",
        "x ~y + ~t ~x y",
        "x ~y + ~t ~x",
        "x (~t y + t ~y)",
        "x (~t y + t ~y)",
        "~t y + t x ~y",
        "~t y + t x ~y",
        "x (~t + ~y)",
        "x (~t + ~y)",
        "x ~y + ~t y",
        "~t + x ~y",
        "t ~y",
        "t ~y",
        "t ~y + ~t ~x y",
        "t ~y + ~t ~x",
        "~y (t + x)",
        "~y",
        "~y (t + x) + ~t ~x y",
        "~y + ~t ~x",
        "t ~y + ~t x y",
        "t ~y + ~t x y",
        "t ~y + ~t y",
        "t ~y + ~t y",
        "t ~y + ~t x",
        "~y + ~t x",
        "~(t y + ~t ~x ~y)",
        "~(t y)",
        "t (~x y + x ~y)",
        "t (~x y + x ~y)",
        "~x y + t x ~y",
        "~x y + t x ~y",
        "x ~y + t ~x y",
        "x ~y + t ~x y",
        "x ~y + ~x y",
        "x ~y + ~x y",
        "t (~x y + x ~y) + ~t x y",
        "t (~x y + x ~y) + ~t x y",
        "y (~t + ~x) + t x ~y",
        "y (~t + ~x) + t x ~y",
        "x (~t + ~y) + t ~x y",
        "x (~t + ~y) + t ~x y",
        "~(~x ~y + t x y)",
        "~(~x ~y + t x y)",
        "t (~x + ~y)",
        "t (~x + ~y)",
        "t ~y + ~x y",
        "~x + t ~y",
        "x ~y + t ~x",
        "~y + t ~x",
        "~(x y + ~t ~x ~y)",
        "~(x y)",
        "t (~x + ~y) + ~t x y",
        "t (~x + ~y) + ~t x y",
        "~(~t ~y + t x y)",
        "~(~t ~y + t x y)",
        "~(~t ~x + t x y)",
        "~(~t ~x + t x y)",
        "~(~t ~x ~y + t x y)",
        "~(t x y)",
        "t x y",
        "~t ~x ~y + t x y",
        "~t ~x + t x y",
        "~t ~x + t x y",
        "~t ~y + t x y",
        "~t ~y + t x y",
        "~t (~x + ~y) + t x y",
        "~(t (~x + ~y) + ~t x y)",
        "x y",
        "x y + ~t ~x ~y",
        "y (~t + x)",
        "~(x ~y + t ~x)",
        "~t ~y + x y",
        "~(t ~y + ~x y)",
        "~t + x y",
        "~(t (~x + ~y))",
        "~x ~y + t x y",
        "~x ~y + t x y",
        "~x (~t + ~y) + t x y",
        "~(x (~t + ~y) + t ~x y)",
        "~y (~t + ~x) + t x y",
        "~(y (~t + ~x) + t x ~y)",
        "~(t (~x y + x ~y) + ~t x y)",
        "~(t (~x y + x ~y) + ~t x y)",
        "~x ~y + x y",
        "~(x ~y + ~x y)",
        "~(x ~y + t ~x y)",
        "~(x ~y + t ~x y)",
        "~(~x y + t x ~y)",
        "~(~x y + t x ~y)",
        "~(t (~x y + x ~y))",
        "~(t (~x y + x ~y))",
        "t y",
        "t y + ~t ~x ~y",
        "y (t + ~x)",
        "~(t ~y + ~t x)",
        "~t ~y + t y",
        "~(t ~y + ~t y)",
        "~(t ~y + ~t x y)",
        "~(t ~y + ~t x y)",
        "y (t + x)",
        "~(~y (t + x) + ~t ~x y)",
        "y",
        "~(~y (t + x))",
        "~(t ~y + ~t ~x)",
        "~(t ~y + ~t ~x y)",
        "~(t ~y)",
        "~(t ~y)",
        "~x ~y + t y",
        "~(x ~y + ~t y)",
        "~x + t y",
        "~(x (~t + ~y))",
        "~(~t y + t x ~y)",
        "~(~t y + t x ~y)",
        "~(x (~t y + t ~y))",
        "~(x (~t y + t ~y))",
        "x y + t ~x",
        "~(x ~y + ~t ~x y)",
        "~(x ~y)",
        "~(x ~y)",
        "~(~t ~x + t x ~y)",
        "~(t x ~y + ~t ~x y)",
        "~(t x ~y)",
        "~(t x ~y)",
        "t x",
        "t x + ~t ~x ~y",
        "~t ~x + t x",
        "~(t ~x + ~t x)",
        "x (t + ~y)",
        "~(~t y + t ~x)",
        "~(t ~x + ~t x y)",
        "~(t ~x + ~t x y)",
        "x (t + y)",
        "~(~x (t + y) + ~t x ~y)",
        "~t y + t x",
        "~(t ~x + ~t x ~y)",
        "x",
        "~(~x (t + y))",
        "~(t ~x)",
        "~(t ~x)",
        "~x ~y + t x",
        "~(~x y + ~t x)",
        "~(~t x + t ~x y)",
        "~(~t x + t ~x y)",
        "~y + t x",
        "~(y (~t + ~x))",
        "~(y (~t x + t ~x))",
        "~(y (~t x + t ~x))",
        "t ~y + x y",
        "~(~x y + ~t x ~y)",
        "~(~t ~y + t ~x y)",
        "~(~t x ~y + t ~x y)",
        "~(~x y)",
        "~(~x y)",
        "~(t ~x y)",
        "~(t ~x y)",
        "t (x + y)",
        "~(~t (x + y) + t ~x ~y)",
        "~x y + t x",
        "~(~t x + t ~x ~y)",
        "x ~y + t y",
        "~(~t y + t ~x ~y)",
        "~(~x ~y + ~t x y)",
        "~(t ~x ~y + ~t x y)",
        "t (x + y) + x y",
        "~(~t (~x y + x ~y) + t ~x ~y)",
        "y + t x",
        "~(~y (~t x + t ~x))",
        "x + t y",
        "~(~x (~t y + t ~y))",
        "~(~x ~y)",
        "~(t ~x ~y)",
        "t",
        "~(~t (x + y))",
        "~(~t x)",
        "~(~t x)",
        "~(~t y)",
        "~(~t y)",
        "~(~t x y)",
        "~(~t x y)",
        "t + x y",
        "~(~t (~x y + x ~y))",
        "~(~t ~y)",
        "~(~t x ~y)",
        "~(~t ~x)",
        "~(~t ~x y)",
        "1",
        "Constant 1"
    };
}
